;DOS-Mode driver for the Microsoft Sidewinder Gamepad
;(C) 1997 Robert William Grubbs, All Rights Reserved
;Latest revision 10/15/97
;  Single Controller only
;  Driver Source code Include file

;Model irrelevent
.386 is not required
;
; Usage:  INCLUDE SWDRV.ASM
;   call SWStatus

;You should make sure the SW is really present before trying to use it


;The main subroutine;  this is the important one;  bow down before it
;IN: none
;Out: bx=buttons  (bit 0=null 1=up 2=dn 3=rt 4=lt 5=A 6=B 7=C 8=X)
;                 (9=Y 10=Z 11=L 12=R 13=St 14=M 15=Parity)
;No registers destroyed (besides BX, of course)
gDump db 100h DUP (0)   ;SW Status dump buffer (space for 256 bytes, uses 200)
bDump db 10h DUP (0)    ;buffer to hold button data for mode B
SWStatus PROC
  push ax
  push cx
  push dx
  push si
  push di
  push bp

  ;implementation note:  interrupt processing may mess up the SW data,
  ; so it may be necessary to disable interrupts during the SW dump below
  ;I havn't had problems so far, but just in case, I thought I'd warn you...

  mov cx,200                  ;dump buffer fill size
  mov bx,0                    ;initial dump pointer
  mov dx,0201h                ;joystick port
  ;cli                         ;uncomment to disble interrupts
  GetSWDataLoop:              ;
  out dx,al                   ;trigger joystick port
  in al,dx                    ;read SW status byte
  mov cs:[gDump+bx],al        ;dump status byte
  inc bx                      ;increment dump pointer
  loop GetSWDataLoop          ;get next status byte (quickly)
  ;sti                         ;uncomment to disble interrupts

  mov cx,0                    ;tick count
  mov si,1                    ;initialize output mask
  mov bx,0                    ;initialize output
  mov di,0                    ;initialize input pointer

  ;My current method of cycle detection is to look for 15 highs in a row on
  ; the strobe line.  Cycle ends is detected by 15 lows in a row.
  ;Mode A has 15 strobes in a cycle, Mode B has 5.
  ; Note that the 15 highs/lows for cycle detection may be too high for slow
  ; machines.  I havn't seen a problem yet, but it may exist...

  FindCycle:
  mov al,cs:[gDump+di]        ;get next status byte
  inc di                      ;increment input pointer
  cmp di,200                  ;test for end of status block
  je SHORT SWNoFind           ;if it's the end, quit sub with error
  test al,00010000b           ;Check for nonzero bits
  jnz SHORT WMFCS1            ;
  xor cx,cx                   ;if zero, reset tick count
  jmp SHORT FindCycle         ;can't be pre-cycle
  WMFCS1:                     ;Possibly pre-cycle
  inc cx                      ;increment tick count
  cmp cx,15                   ;test for sufficient ticks for cycle start
  jne SHORT FindCycle         ;if insufficient, get next status byte
                              ;Yippie! it found a (probable) cycle!

  mov bp,0                    ;initialize bDump index

  FindStrobeLow:              ;Search for leading edge of data strobe
  mov al,cs:[gDump+di]        ;get next status byte
  inc di                      ;increment input pointer
  cmp di,200                  ;test for end of status block
  je SHORT SWNoFind           ;if it's the end, quit sub with error
  test al,00010000b           ;get "strobe" bit
  jnz SHORT FindStrobeLow     ;if it isn't zero, we're not there yet
  xor cx,cx                   ;initialize cycle end test count

  FindStrobeHigh:
  inc cx                      ;increment zero count
  cmp cx,0fh                  ;is it 15?
  je SWModeBCheck             ;if so, goto mode B check
  mov al,cs:[gDump+di]        ;get next status byte
  inc di                      ;increment input pointer
  cmp di,200                  ;test for end of status block
  je SHORT SWNoFind           ;if it's the end, quit sub with error
  test al,00010000b           ;get "strobe" bit
  jz SHORT FindStrobeHigh     ;if it is zero, we're not there yet
                              ;if not, we're there!  data bit is valid (probably)
  mov CS:[bDump+bp],al        ;preserve data for mode B
  inc bp                      ;increment cycle count/bDump index
  shl si,1                    ;set next output bit (mode A)
  test al,00100000b           ;test for button press (mode A)
  jnz SHORT NoButton          ;Button not pressed, so wait for next button
  or bx,si                    ;Button IS pressed, or the bit into the output
  NoButton:
  cmp bp,15                   ;Is Mode A done?
  je SHORT SMWDone            ;
  jmp SHORT FindStrobeLow     ;if it isn't, wait for the next button

  SMWDone:
  pop bp
  pop di
  pop si
  pop dx
  pop cx
  pop ax                      ;restore registers
  clc                         ;indicate no error
  ret                         ;return to calling procedure
  SWNoFind:
  ;error return.  Calling sub should re-dump status, or report "no sidewinder"
  pop bp
  pop di
  pop si
  pop dx
  pop cx
  pop ax                      ;restore registers
  stc                         ;indicate error
  ret
  SWModeBCheck:
  cmp bp,5                    ;Check strobe count (Mode B always has five)
  jne SHORT SWNoFind
  ;Handle mode B buttons
  mov bx,0                    ;Initialize output
  test CS:[bDump],00100000b   ;Check bit 5 of Cycle 0
  jnz NoUp                    ;if zero, up is not pressed
  or bx,0000000000000010b     ;set Up bit in output
  NoUp:                       ;
  test CS:[bDump],01000000b   ;continues the same for all buttons.
  jnz NoDn
  or bx,0000000000000100b
  NoDn:
  test CS:[bDump],10000000b
  jnz NoRt
  or bx,0000000000001000b
  NoRt:
  test CS:[bDump+1],00100000b
  jnz NoLt
  or bx,0000000000010000b
  NoLt:
  test CS:[bDump+1],01000000b
  jnz NoA
  or bx,0000000000100000b
  NoA:
  test CS:[bDump+1],10000000b
  jnz NoB
  or bx,0000000001000000b
  NoB:
  test CS:[bDump+2],00100000b
  jnz NoC
  or bx,0000000010000000b
  NoC:
  test CS:[bDump+2],01000000b
  jnz NoX
  or bx,0000000100000000b
  NoX:
  test CS:[bDump+2],10000000b
  jnz NoY
  or bx,0000001000000000b
  NoY:
  test CS:[bDump+3],00100000b
  jnz NoZ
  or bx,0000010000000000b
  NoZ:
  test CS:[bDump+3],01000000b
  jnz NoL
  or bx,0000100000000000b
  NoL:
  test CS:[bDump+3],10000000b
  jnz NoR
  or bx,0001000000000000b
  NoR:
  test CS:[bDump+4],00100000b
  jnz NoSt
  or bx,0010000000000000b
  NoSt:
  test CS:[bDump+4],01000000b
  jnz NoM
  or bx,0100000000000000b
  NoM:
  test CS:[bDump+4],10000000b
  jnz NoParity
  or bx,1000000000000000b
  NoParity:
  jmp SMWDone
ENDP

